////////////////////////////////////////////////////////////////////////////////
//
// Fracter - A simple Mandelbrot Set viewer.
//
// Copyright (C) 2010 Sarah Eris Horsley Caffee
//
// This file is part of Fracter.
//
// Fracter is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.
//
//
// Mandelbrot.cpp
// Mandelbrot set calculator class implementation
//
////////////////////////////////////////////////////////////////////////////////

#include "Mandelbrot.h"
#include <typeinfo>

////////////////////////////////////////////////////////////////////////////////
ViewMandelbrot::ViewMandelbrot(std::string t, 
			       double re, double im, double s, 
			       unsigned int max) : 
   View(t, re, im, s, max)
   {
   }

////////////////////////////////////////////////////////////////////////////////

Mandelbrot::Mandelbrot()
   {
   re_min = -2.5;
   im_max = 2.5;
   size = 5.0;
   max_iter = 50;
   calc_needed = true;
   fractal_name = "Mandelbrot set";
   display_name = fractal_name;
   }


////////////////////////////////////////////////////////////////////////////////

bool Mandelbrot::calc_set()
   {
   if (!surface) return false;

   // See http://warp.povusers.org/Mandelbrot/ for explanation of the algorithm.

   if (calc_needed)
      {
      double re_scale = (size)/(surface->w-1);
      double im_scale = (size)/(surface->h-1);

      int r, g, b;

      Uint32 black = SDL_MapRGB(surface->format, 0, 0, 0);
      Uint32 color;
      int halfway = max_iter/2 - 1;
      float step = (float) 255 / halfway;

      int x, y, n;

      for(y = 0; y < surface->h; ++y)
	 {
	 double c_im = im_max - y * im_scale;
	 for(x = 0; x < surface->w; ++x)
	    {
	    double c_re = re_min + x * re_scale;

	    double z_re = c_re, z_im = c_im;
	    bool in_set = true;
	    for(n=0; n < (int) max_iter; ++n)
	       {
	       double z_re2 = z_re * z_re, z_im2 = z_im * z_im;
	       if(z_re2 + z_im2 > 4)
		  {
		  in_set = false;
		  break;
		  }
	       z_im = 2 * z_re * z_im + c_im;
	       z_re = z_re2 - z_im2 + c_re;
	       }
	    if(in_set) 
	       { 
	       draw_pixel(x, y, &black); 
	       }
	    else
	       {
	       if (n <= halfway)
		  {
		  b = n * step;
		  r = g = 0;
		  }
	       else 
		  {
		  b = 255;
		  r = g = (n - halfway) * step;
		  }
	       color = SDL_MapRGB(surface->format, r, g, b);
	       draw_pixel(x, y, &color); 
	       }
	    }
	 }
      }

   calc_needed = false;
   return true;
   }


////////////////////////////////////////////////////////////////////////////////
ViewMandelbrot * Mandelbrot::save_view()
   {
   return new ViewMandelbrot(typeid(*this).name(), re_min, im_max, size, 
			     max_iter);
   }

////////////////////////////////////////////////////////////////////////////////
void Mandelbrot::restore_view(View * v)
   {
   Fractal::restore_view(v);
   }

